from django.contrib import admin
from django.db.models import Q
from edc_crf.modeladmin_mixins import CrfModelAdmin

from ..admin_site import edc_he_admin
from ..forms import HealthEconomicsPatientForm
from ..modeladmin_mixins import HealthEconomicsPatientModelAdminMixin
from ..models import Ethnicities, HealthEconomicsPatient, Religions


@admin.register(HealthEconomicsPatient, site=edc_he_admin)
class HealthEconomicsPatientAdmin(HealthEconomicsPatientModelAdminMixin, CrfModelAdmin):
    form = HealthEconomicsPatientForm

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if getattr(request, "site", None):
            if db_field.name == "pat_ethnicity":
                kwargs["queryset"] = Ethnicities.objects.filter(
                    Q(extra_value=request.site.siteprofile.country)
                    | Q(extra_value__isnull=True)
                )
            if db_field.name == "pat_religion":
                kwargs["queryset"] = Religions.objects.filter(
                    Q(extra_value=request.site.siteprofile.country)
                    | Q(extra_value__isnull=True)
                )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if getattr(request, "site", None) and db_field.name == "pat_insurance":
            model_cls = getattr(self.model, db_field.name).field.related_model
            kwargs["queryset"] = model_cls.objects.filter(
                Q(extra_value=request.site.siteprofile.country) | Q(extra_value__isnull=True)
            )
        return super().formfield_for_manytomany(db_field, request, **kwargs)



from clinicedc_constants import YES
from django import forms
from django.core.exceptions import ObjectDoesNotExist

from ..utils import (
    get_assets_model_cls,
    get_household_head_model_cls,
    get_income_model_cls,
    get_patient_model_cls,
    get_property_model_cls,
)


class HealthEconomicsModelFormMixin:
    def clean(self):
        self.raise_if_he_household_head_required()
        self.raise_if_he_patient_required()
        self.raise_if_he_assets_required()
        self.raise_if_he_property_required()
        return super().clean()

    @property
    def household_head(self):
        return get_household_head_model_cls().objects.get(
            subject_visit__subject_identifier=self.get_subject_identifier()
        )

    def raise_if_he_household_head_required(self):
        if self._meta.model != get_household_head_model_cls():
            try:
                self.household_head  # noqa: B018
            except ObjectDoesNotExist as e:
                raise forms.ValidationError(
                    f"Complete {get_household_head_model_cls()._meta.verbose_name} CRF first."
                ) from e

    def raise_if_he_patient_required(self):
        """Raise if patient is the head of household and the Patient
        form/instance has not yet been completed.

        Patient is HoH if self.household_head.hoh == YES.
        """
        if self.household_head.hoh == YES and self._meta.model in [
            get_assets_model_cls(),
            get_income_model_cls(),
            get_property_model_cls(),
        ]:
            try:
                get_patient_model_cls().objects.get(
                    subject_visit__subject_identifier=self.get_subject_identifier()
                )
            except ObjectDoesNotExist as e:
                raise forms.ValidationError(
                    f"Complete {get_patient_model_cls()._meta.verbose_name} CRF first."
                ) from e

    def raise_if_he_assets_required(self):
        if self._meta.model in [
            get_income_model_cls(),
            get_property_model_cls(),
        ]:
            try:
                get_assets_model_cls().objects.get(
                    subject_visit__subject_identifier=self.get_subject_identifier()
                )
            except ObjectDoesNotExist as e:
                raise forms.ValidationError(
                    f"Complete {get_assets_model_cls()._meta.verbose_name} CRF first."
                ) from e

    def raise_if_he_property_required(self):
        if self._meta.model in [
            get_income_model_cls(),
        ]:
            try:
                get_property_model_cls().objects.get(
                    subject_visit__subject_identifier=self.get_subject_identifier()
                )
            except ObjectDoesNotExist as e:
                raise forms.ValidationError(
                    f"Complete {get_property_model_cls()._meta.verbose_name} CRF first."
                ) from e



# Generated by Django 5.2.6 on 2025-10-01 02:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_he", "0009_rename_education_educationtype_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="HistoricalEducationType",
        ),
    ]



from clinicedc_constants import OPTION_RETIRED
from django.contrib import admin
from edc_crf.modeladmin_mixins import CrfModelAdmin

from ..admin_site import edc_he_admin
from ..choices import EXTERNAL_WALL_MATERIALS_CHOICES
from ..forms import HealthEconomicsAssetsForm
from ..modeladmin_mixins import HealthEconomicsAssetsModelAdminMixin
from ..models import HealthEconomicsAssets


@admin.register(HealthEconomicsAssets, site=edc_he_admin)
class HealthEconomicsAssetsAdmin(
    HealthEconomicsAssetsModelAdminMixin,
    CrfModelAdmin,
):
    form = HealthEconomicsAssetsForm

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "external_wall_material":
            kwargs["choices"] = self.external_wall_material_choices
        return super().formfield_for_choice_field(db_field, request, **kwargs)

    @property
    def external_wall_material_choices(self) -> tuple[tuple[str, str]]:
        return tuple(
            [tpl for tpl in EXTERNAL_WALL_MATERIALS_CHOICES if tpl[0] != OPTION_RETIRED]
        )



from .assets_form import HealthEconomicsAssetsForm
from .education_form import EducationForm
from .household_head import HealthEconomicsHouseholdHeadForm
from .income_form import HealthEconomicsIncomeForm
from .modelform_mixins import HealthEconomicsModelFormMixin
from .patient_form import HealthEconomicsPatientForm
from .property_form import HealthEconomicsPropertyForm



from django.contrib import admin
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple


def get_income_fieldsets_tuple() -> list[tuple]:
    fieldsets = []
    sources = [
        ("wages", _("Wages / Salary")),
        ("selling", _("Selling, trading or hawking")),
        ("rental_income", _("Rental income")),
        ("pension", _("Pension, etc.")),
        ("ngo_assistance", _("NGO assistance")),
        ("interest", _("Interest and dividends")),
        ("internal_remit", _("Money transfers (domestic)")),
        ("external_remit", _("Remittances (from outside the country)")),
        ("more_sources", _("Other sources of income")),
    ]
    for fld, label in sources:
        fields = [
            fld,
            f"{fld}_value_known",
            f"{fld}_value",
        ]
        if fld == "external_remit":
            fields.extend(["external_remit_currency", "external_remit_currency_other"])
        if fld == "more_sources":
            fields.insert(1, "more_sources_other")

        fieldsets.append(
            (
                label,
                {
                    "description": format_html(
                        "{html}",
                        html=mark_safe(  # noqa: S308
                            _(
                                "Estimate the total amount of income from this source "
                                "for the <b>household</b> over the time period indicated"
                            )
                        ),  # nosec B703, B308
                    ),
                    "fields": tuple(fields),
                },
            )
        )
    return fieldsets


class HealthEconomicsIncomeModelAdminMixin:
    form = None

    additional_instructions = format_html(
        "{html}",
        html=mark_safe(  # noqa: S308
            render_to_string("edc_he/income/additional_instructions.html")
        ),
    )

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        *get_income_fieldsets_tuple(),
        (
            "Dependents, financial status",
            {
                "fields": (
                    "external_dependents",
                    "income_enough",
                    "financial_status",
                    "financial_status_compare",
                )
            },
        ),
        (
            "Debt / Loans",
            {
                "description": format_html(
                    "{html}",
                    html=mark_safe(  # noqa: S308
                        render_to_string("edc_he/income/debt_loans.html")
                    ),
                ),
                "fields": (
                    "household_debt",
                    "household_debt_value",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {  # noqa: RUF012
        "wages": admin.VERTICAL,
        "wages_value_known": admin.VERTICAL,
        "selling": admin.VERTICAL,
        "selling_value_known": admin.VERTICAL,
        "rental_income": admin.VERTICAL,
        "rental_income_value_known": admin.VERTICAL,
        "pension": admin.VERTICAL,
        "pension_value_known": admin.VERTICAL,
        "ngo_assistance": admin.VERTICAL,
        "ngo_assistance_value_known": admin.VERTICAL,
        "interest": admin.VERTICAL,
        "interest_value_known": admin.VERTICAL,
        "internal_remit": admin.VERTICAL,
        "internal_remit_value_known": admin.VERTICAL,
        "external_remit": admin.VERTICAL,
        "external_remit_value_known": admin.VERTICAL,
        "external_remit_currency": admin.VERTICAL,
        "more_sources": admin.VERTICAL,
        "more_sources_value_known": admin.VERTICAL,
        "income_enough": admin.VERTICAL,
        "financial_status": admin.VERTICAL,
        "financial_status_compare": admin.VERTICAL,
        "household_debt": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }



from clinicedc_constants import NOT_APPLICABLE, QUESTION_RETIRED
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _
from edc_constants.choices import YES_NO, YES_NO_DONT_KNOW_DWTA_NA
from edc_model_fields.fields import OtherCharField

from ..choices import (
    COOKING_FUEL_CHOICES,
    EXTERNAL_WALL_MATERIALS_CHOICES,
    FLOOR_MATERIALS_CHOICES,
    LIGHTING_CHOICES,
    RESIDENCE_OWNERSHIP_CHOICES,
    ROOF_MATERIAL_CHOICES,
    TOILET_CHOICES,
    WATER_OBTAIN_CHOICES,
    WATER_SOURCE_CHOICES,
    WINDOW_MATERIAL_CHOICES,
)


class AssetsModelMixin(models.Model):
    residence_ownership = models.CharField(
        verbose_name=_(
            "Is the house you live in rented, owned by you (either on your own, or "
            "with someone else), or owned by someone else in your family?"
        ),
        max_length=25,
        choices=RESIDENCE_OWNERSHIP_CHOICES,
    )

    dwelling_value_known = models.CharField(
        verbose_name=_(
            "If the owner [you] were to sell this dwelling today, do you know the "
            "approximate value?"
        ),
        max_length=25,
        choices=YES_NO_DONT_KNOW_DWTA_NA,
        default=NOT_APPLICABLE,
        help_text="in local currency",
    )

    dwelling_value = models.IntegerField(
        verbose_name=_("About how much is it worth?"),
        validators=[MinValueValidator(0), MaxValueValidator(9999999999)],
        null=True,
        blank=True,
        help_text="in local currency",
    )

    rooms = models.IntegerField(
        verbose_name=_(
            "How many rooms does your dwelling have in total, without counting "
            "the bathrooms/ toilets or hallways/passageways?"
        ),
        validators=[MinValueValidator(1), MaxValueValidator(30)],
    )

    bedrooms = models.IntegerField(
        verbose_name=_("How many rooms are used for sleeping in your dwelling?"),
        validators=[MinValueValidator(0), MaxValueValidator(30)],
    )

    beds = models.IntegerField(
        verbose_name=_("How many beds does your dwelling have in total?"),
        validators=[MinValueValidator(0), MaxValueValidator(30)],
    )

    water_source = models.CharField(
        verbose_name=_("What is the main source of drinking water for the household?"),
        max_length=25,
        choices=WATER_SOURCE_CHOICES,
    )

    water_source_other = OtherCharField(verbose_name=_("If OTHER water source, specify"))

    water_obtain_time = models.CharField(
        verbose_name=_("How long does it take to obtain drinking water?"),
        max_length=25,
        choices=WATER_OBTAIN_CHOICES,
    )

    # TODO: change to M2M
    toilet = models.CharField(
        verbose_name=_("What type of toilet is mainly used in your household?"),
        max_length=25,
        choices=TOILET_CHOICES,
        help_text=_(
            "Note: 'private' (1-7) is a toilet solely used by the household and 'shared' "
            "(8-14) is a toilet shared by two or more households."
        ),
    )

    toilet_other = OtherCharField(verbose_name=_("If OTHER type of toilet, specify"))

    roof_material = models.CharField(
        verbose_name=_("What is the major construction material of the roof?"),
        max_length=25,
        choices=ROOF_MATERIAL_CHOICES,
    )

    roof_material_other = OtherCharField(verbose_name=_("If OTHER roof material, specify"))

    eaves = models.CharField(
        verbose_name=_("Are the eaves open, partially or fully closed?"),
        max_length=25,
        default=QUESTION_RETIRED,
    )

    external_wall_material = models.CharField(
        verbose_name=_("What is the major construction material of the external wall?"),
        max_length=25,
        choices=EXTERNAL_WALL_MATERIALS_CHOICES,
    )

    external_wall_material_other = OtherCharField(
        verbose_name=_("If OTHER external wall material, specify"),
    )

    external_window_material = models.CharField(
        verbose_name=_("What is the main material on external windows?"),
        max_length=25,
        choices=WINDOW_MATERIAL_CHOICES,
    )

    external_window_material_other = OtherCharField(
        verbose_name=_("If OTHER external window material, specify"),
    )

    window_screens = models.CharField(
        verbose_name=_("What is the main screening material of external windows?"),
        max_length=25,
        default=QUESTION_RETIRED,
    )

    window_screen_type = models.CharField(
        verbose_name=_("Type of screening on external windows"),
        max_length=25,
        default=QUESTION_RETIRED,
    )

    floor_material = models.CharField(
        verbose_name=_("What is the major construction material of the floor?"),
        max_length=25,
        choices=FLOOR_MATERIALS_CHOICES,
        help_text=(
            "If there are similar amounts of more than one type of flooring, "
            "select the multiple types that apply. Otherwise select the main "
            "type of flooring"
        ),
    )

    floor_material_other = OtherCharField(
        verbose_name=_("If OTHER floor material, specify"),
    )

    electricity = models.CharField(
        verbose_name=_("Does your household have electricity?"),
        max_length=25,
        choices=YES_NO,
    )

    lighting_source = models.CharField(
        verbose_name=_("What is the main source of lighting?"),
        max_length=25,
        choices=LIGHTING_CHOICES,
    )

    lighting_source_other = OtherCharField(
        verbose_name=_("If OTHER lighting source, specify"),
    )

    cooking_fuel = models.CharField(
        verbose_name=_("What type of fuel do you primarily use for cooking?"),
        max_length=25,
        choices=COOKING_FUEL_CHOICES,
    )

    cooking_fuel_other = OtherCharField(
        verbose_name=_("If OTHER cooking fuel, specify"),
    )

    class Meta:
        abstract = True



from clinicedc_constants import QUESTION_RETIRED
from django.db import models
from django.db.models import PROTECT
from django.utils.translation import gettext_lazy as _
from edc_model_fields.fields import OtherCharField

from ..choices import EMPLOYMENT_STATUS_CHOICES, MARITAL_CHOICES


class PatientModelMixin(models.Model):
    pat_religion = models.ForeignKey(
        "edc_he.religions",
        verbose_name=_("How would you describe your religious orientation?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
    )

    pat_religion_other = OtherCharField(
        verbose_name=_("If OTHER religious orientation, specify ..."),
    )

    pat_ethnicity = models.ForeignKey(
        "edc_he.ethnicities",
        verbose_name=_("What is your ethnic background?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
    )

    pat_ethnicity_other = OtherCharField(
        verbose_name=_("If OTHER ethnic background, specify ..."),
    )

    pat_education = models.ForeignKey(
        "edc_he.educationtype",
        verbose_name=_("Highest level of education completed?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
    )

    pat_education_other = OtherCharField(
        verbose_name=_("If OTHER level of education, specify ..."),
    )

    pat_employment_status = models.CharField(
        verbose_name=_("What is your employment status?"),
        max_length=25,
        choices=EMPLOYMENT_STATUS_CHOICES,
    )

    pat_employment_type = models.ForeignKey(
        "edc_he.employmenttype",
        verbose_name=_("What is your type of employment?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
    )

    pat_employment_type_other = OtherCharField(
        verbose_name=_("If OTHER type of employment, specify ..."),
        max_length=100,
    )

    pat_marital_status = models.CharField(
        verbose_name=_("What is your marital status?"),
        max_length=25,
        choices=MARITAL_CHOICES,
    )
    pat_marital_status_other = OtherCharField(
        verbose_name=_("If OTHER marital status, specify ..."),
    )

    pat_insurance = models.ManyToManyField(
        "edc_he.insurancetypes",
        verbose_name=_("What is your health insurance status?"),
        related_name="+",
    )

    pat_insurance_other = OtherCharField(
        verbose_name=_("If OTHER health insurance status, specify ..."),
    )

    # not used
    pat_ethnicity_old = models.CharField(
        verbose_name=_("What is your ethnic background?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    # not used
    pat_employment_type_old = models.CharField(
        verbose_name=_("What is your type of employment?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    # not used
    pat_education_old = models.CharField(
        verbose_name=_("Highest level of education completed?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    # not used
    pat_religion_old = models.CharField(
        verbose_name=_("How would you describe your religious orientation?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    class Meta:
        abstract = True



from django.utils.translation.trans_null import gettext as _
from edc_crf.model_mixins import CrfModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import EducationModelMixin


class Education(EducationModelMixin, CrfModelMixin, BaseUuidModel):
    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = _("Health Economics: Education")
        verbose_name_plural = _("Health Economics: Education")
        indexes = (*CrfModelMixin.Meta.indexes, *BaseUuidModel.Meta.indexes)



from __future__ import annotations

from typing import TYPE_CHECKING

from clinicedc_constants import NOT_APPLICABLE, QUESTION_RETIRED
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _
from edc_constants.choices import YES_NO_DONT_KNOW_DWTA

from edc_he.calculators import convert_to_sq_meters
from edc_he.choices import LAND_AREA_UNITS

if TYPE_CHECKING:
    from decimal import Decimal


class PropertyModelMixin(models.Model):
    land_owner = models.CharField(
        verbose_name=_("Do you own any land."),
        max_length=25,
        choices=YES_NO_DONT_KNOW_DWTA,
    )

    # QUESTION_RETIRED
    land_value_known = models.CharField(
        verbose_name=_("Do you know about how much is this worth in total?"),
        max_length=25,
        default=QUESTION_RETIRED,
        help_text=_("Use cash equivalent in local currency"),
    )

    land_value = models.IntegerField(
        verbose_name=_("About how much is this worth in total?"),
        validators=[MinValueValidator(1), MaxValueValidator(999999999)],
        null=True,
        blank=True,
        help_text=_("Use cash equivalent in local currency"),
    )

    land_surface_area = models.DecimalField(
        verbose_name=_("Surface area"),
        decimal_places=1,
        max_digits=15,
        validators=[MinValueValidator(0.1), MaxValueValidator(999999999.9)],
        null=True,
        blank=True,
    )

    land_surface_area_units = models.CharField(
        verbose_name=_("Surface area (units)"),
        max_length=15,
        choices=LAND_AREA_UNITS,
        default=NOT_APPLICABLE,
    )

    land_additional = models.CharField(
        verbose_name=_("Do you own any other property other than your primary dwelling?"),
        max_length=25,
        choices=YES_NO_DONT_KNOW_DWTA,
    )

    # QUESTION_RETIRED
    land_additional_known = models.CharField(
        verbose_name=_("Do you know about how much is this worth in total?"),
        max_length=25,
        default=QUESTION_RETIRED,
        help_text=_("Use cash equivalent in local currency"),
    )

    land_additional_value = models.IntegerField(
        verbose_name=_("About how much is this worth in total?"),
        validators=[MinValueValidator(1), MaxValueValidator(999999999)],
        null=True,
        blank=True,
        help_text=_("Use cash equivalent in local currency"),
    )

    calculated_land_surface_area = models.DecimalField(
        decimal_places=2,
        max_digits=15,
        validators=[MinValueValidator(0), MaxValueValidator(999999999.99)],
        null=True,
        blank=True,
        help_text="m2 (system calculated)",
    )

    def save(self, *args, **kwargs):
        self.calculated_land_surface_area = self.get_calculated_land_surface_area()
        super().save(*args, **kwargs)

    def get_calculated_land_surface_area(self) -> Decimal | None:
        """Returns land surface area converted to m2."""
        calculated_land_surface_area = None
        if self.land_surface_area and self.land_surface_area_units:
            calculated_land_surface_area = convert_to_sq_meters(
                area=self.land_surface_area,
                area_units=self.land_surface_area_units,
            )
        return calculated_land_surface_area

    class Meta:  # noqa: DJ012
        abstract = True



# Generated by Django 5.2.6 on 2025-10-01 02:24

import django.db.models.deletion
import simple_history.models
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_he", "0011_historicaleducation_education"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalEducationType",
            fields=[
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="This is the stored value, required",
                        max_length=250,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(
                        default="", max_length=250, verbose_name="Plural name"
                    ),
                ),
                (
                    "display_name",
                    models.CharField(
                        db_index=True,
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        default="",
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                    ),
                ),
                ("extra_value", models.CharField(default="", max_length=250)),
                (
                    "version",
                    models.CharField(default="1.0", editable=False, max_length=35),
                ),
                ("id", models.IntegerField(blank=True, db_index=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Education type",
                "verbose_name_plural": "historical Education types",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from __future__ import annotations

from decimal import Decimal

from .constants import ACRES, DECIMALS, HECTARES, SQ_FEET, SQ_METERS


class InvalidAreaUnitsError(Exception):
    pass


def convert_to_sq_meters(area: int | Decimal | float, area_units: str) -> Decimal:
    """Returns the area converted to square meters.

    Raises InvalidAreaUnitsError if `area_units` value not recognised.
    """
    if area_units == SQ_METERS:
        return Decimal(str(area))
    if area_units == ACRES:
        return acres_to_sq_meters(area)
    if area_units == DECIMALS:
        return decimals_to_sq_meters(area)
    if area_units == HECTARES:
        return hectares_to_sq_meters(area)
    if area_units == SQ_FEET:
        return sq_feet_to_sq_meters(area)
    raise InvalidAreaUnitsError(
        "Invalid. Area units not currently supported. "
        "Either modify argument or define a conversion to square meters. "
        f"Got area_units='{area_units}'"
    )


def acres_to_sq_meters(area: int | Decimal | float) -> Decimal:
    # International foot metric equivalent (exact): 1 acre = 4046.8564224 m2

    # Conversion source:
    #   - https://www.nist.gov/pml/us-surveyfoot/revised-unit-conversion-factors

    # NOTE:
    # "Beginning on January 1, 2023, the U.S. survey foot should be avoided
    # [...] and has been superseded by the international foot definition
    # (i.e., 1 foot = 0.3048 meter exactly)"
    # - https://www.nist.gov/pml/us-surveyfoot
    return Decimal(str(area)) * Decimal("4046.8564224")


def hectares_to_sq_meters(area: int | Decimal | float) -> Decimal:
    # Conversion sources:
    # - https://www.nist.gov/pml/owm/metric-si/si-units-area
    # - https://www.nist.gov/pml/special-publication-811/nist-guide-si-appendix-b-conversion-factors/nist-guide-si-appendix-b9#AREA  # noqa
    return Decimal(str(area)) * Decimal("1E04")


def decimals_to_sq_meters(area: int | Decimal | float) -> Decimal:
    return acres_to_sq_meters(area) / Decimal(100)


def sq_feet_to_sq_meters(area: int | Decimal | float) -> Decimal:
    # Conversion sources:
    # - https://www.nist.gov/pml/special-publication-811/nist-guide-si-appendix-b-conversion-factors/nist-guide-si-appendix-b9#AREA  # noqa
    return Decimal(str(area)) * Decimal("9.290304E-02")



from .assets_model_mixin import AssetsModelMixin
from .education_model_mixin import EducationModelMixin
from .household_head_model_mixin import HouseholdHeadModelMixin
from .household_model_mixin import HouseholdModelMixin
from .income_model_mixin import IncomeModelMixin
from .patient_model_mixin import PatientModelMixin
from .property_model_mixin import PropertyModelMixin

__all__ = [
    "AssetsModelMixin",
    "EducationModelMixin",
    "HouseholdHeadModelMixin",
    "HouseholdModelMixin",
    "IncomeModelMixin",
    "PatientModelMixin",
    "PropertyModelMixin",
]



from clinicedc_constants import NOT_APPLICABLE, QUESTION_RETIRED
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.db.models import PROTECT
from django.utils.translation import gettext_lazy as _
from edc_constants.choices import GENDER_NA, YES_NO
from edc_model_fields.fields import OtherCharField

from ..choices import EMPLOYMENT_STATUS_CHOICES, MARITAL_CHOICES, RELATIONSHIP_CHOICES


class HouseholdHeadModelMixin(models.Model):
    hoh = models.CharField(
        verbose_name=_("Are you the household head?"),
        max_length=15,
        choices=YES_NO,
        help_text=_("If YES, STOP and save the form."),
    )

    hoh_gender = models.CharField(
        verbose_name=_("Is the household head female or male?"),
        max_length=15,
        choices=GENDER_NA,
        default=NOT_APPLICABLE,
    )

    hoh_age = models.IntegerField(
        verbose_name=_("How old is the household head?"),
        validators=[MinValueValidator(18), MaxValueValidator(110)],
        null=True,
        blank=True,
        help_text=_("In years as of today"),
    )

    relationship_to_hoh = models.CharField(
        verbose_name=_("What is your relationship to the household head?"),
        max_length=25,
        choices=RELATIONSHIP_CHOICES,
        default=NOT_APPLICABLE,
        help_text=_("Not applicable if patient is head of household"),
    )

    relationship_to_hoh_other = OtherCharField(
        verbose_name=_("If OTHER relationship, specify ..."),
    )

    hoh_religion = models.ForeignKey(
        "edc_he.Religions",
        verbose_name=_("How would you describe the household head's religious orientation?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text=_("Not applicable if patient is head of household"),
    )

    hoh_religion_other = OtherCharField(
        verbose_name=_("If OTHER religious orientation, specify ..."),
    )

    hoh_ethnicity = models.ForeignKey(
        "edc_he.ethnicities",
        verbose_name=_("What is the household head's ethnic background?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text=_("Not applicable if patient is head of household"),
    )

    hoh_ethnicity_other = OtherCharField(
        verbose_name=_("If OTHER ethnic background, specify ..."),
    )

    hoh_education = models.ForeignKey(
        "edc_he.educationtype",
        verbose_name=_("Highest level of education completed by the household head?"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text=_("Not applicable if patient is head of household"),
    )

    hoh_education_other = OtherCharField(
        verbose_name=_("If OTHER education, specify ..."),
    )

    hoh_employment_status = models.CharField(
        verbose_name=_("Household head's employment status"),
        max_length=25,
        choices=EMPLOYMENT_STATUS_CHOICES,
        default=NOT_APPLICABLE,
        help_text=_(
            "Not applicable if patient is head of household. If the household head has "
            "more than one job, think about their MAIN source of employment"
        ),
    )

    hoh_employment_type = models.ForeignKey(
        "edc_he.employmenttype",
        verbose_name=_("Household head's type of employment"),
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text=_("Not applicable if patient is head of household"),
    )

    hoh_employment_type_other = OtherCharField(
        verbose_name=_("If OTHER type of employment, specify ..."),
        max_length=100,
    )

    hoh_marital_status = models.CharField(
        verbose_name=_("Household head's marital status"),
        max_length=25,
        choices=MARITAL_CHOICES,
        default=NOT_APPLICABLE,
        help_text=_("Not applicable if patient is head of household"),
    )

    hoh_marital_status_other = OtherCharField(
        verbose_name=_("If OTHER marital status, specify ..."),
    )

    hoh_insurance = models.ManyToManyField(
        "edc_he.insurancetypes",
        verbose_name=_("Household head's health insurance and 'club' status "),
        related_name="+",
        help_text=_("Not applicable if patient is head of household"),
    )

    hoh_insurance_other = OtherCharField(
        verbose_name=_("If OTHER, specify ..."),
    )

    # not used
    hoh_religion_old = models.CharField(
        verbose_name=_("How would you describe the household head's religious orientation?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    # not used
    hoh_employment_type_old = models.CharField(
        verbose_name=_("Household head's type of employment"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    # not used
    hoh_education_old = models.CharField(
        verbose_name=_("Highest level of education completed by the household head?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    # not used
    hoh_ethnicity_old = models.CharField(
        verbose_name=_("What is the household head's ethnic background?"),
        max_length=25,
        default=QUESTION_RETIRED,
        editable=False,
    )

    class Meta:
        abstract = True



from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple


class HealthEconomicsAssetsModelAdminMixin:
    form = None

    additional_instructions = _(
        "We want to learn about the household and we use these questions "
        "to get an understanding of wealth and opportunities in the community. "
    )

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            _("Household assets"),
            {
                "fields": (
                    "residence_ownership",
                    "dwelling_value_known",
                    "dwelling_value",
                    "rooms",
                    "bedrooms",
                    "beds",
                    "roof_material",
                    "roof_material_other",
                    "external_wall_material",
                    "external_wall_material_other",
                    "external_window_material",
                    "external_window_material_other",
                    "floor_material",
                    "floor_material_other",
                    "electricity",
                    "lighting_source",
                    "lighting_source_other",
                    "cooking_fuel",
                    "cooking_fuel_other",
                )
            },
        ),
        (
            _("Drinking water and sanitation"),
            {
                "fields": (
                    "water_source",
                    "water_source_other",
                    "water_obtain_time",
                    "toilet",
                    "toilet_other",
                )
            },
        ),
        (
            _("Household assets (continued)"),
            {
                "description": format_html(
                    "{text1}? <BR>{text2}",
                    text1=_(
                        "Does your household or anyone in your household have the following "
                        "in working order"
                    ),
                    text2=_(
                        "Note: If a household owns one of the assets below but the asset "
                        "is not in working order then it should be marked as 'No'"
                    ),
                ),
                "fields": (
                    "radio",
                    "television",
                    "mobile_phone",
                    "computer",
                    "telephone",
                    "fridge",
                    "generator",
                    "iron",
                    "bicycle",
                    "motorcycle",
                    "dala_dala",
                    "car",
                    "motorboat",
                    "large_livestock",
                    "small_animals",
                    "shop",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {  # noqa: RUF012
        "bicycle": admin.VERTICAL,
        "car": admin.VERTICAL,
        "computer": admin.VERTICAL,
        "cooking_fuel": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "dala_dala": admin.VERTICAL,
        "dwelling_value_known": admin.VERTICAL,
        "electricity": admin.VERTICAL,
        "external_wall_material": admin.VERTICAL,
        "external_window_material": admin.VERTICAL,
        "floor_material": admin.VERTICAL,
        "fridge": admin.VERTICAL,
        "generator": admin.VERTICAL,
        "iron": admin.VERTICAL,
        "large_livestock": admin.VERTICAL,
        "lighting_source": admin.VERTICAL,
        "mobile_phone": admin.VERTICAL,
        "motorboat": admin.VERTICAL,
        "motorcycle": admin.VERTICAL,
        "radio": admin.VERTICAL,
        "residence_ownership": admin.VERTICAL,
        "roof_material": admin.VERTICAL,
        "shop": admin.VERTICAL,
        "small_animals": admin.VERTICAL,
        "telephone": admin.VERTICAL,
        "television": admin.VERTICAL,
        "toilet": admin.VERTICAL,
        "water_obtain_time": admin.VERTICAL,
        "water_source": admin.VERTICAL,
    }



from clinicedc_constants import (
    DONT_KNOW,
    DWTA,
    MONTHLY,
    NOT_APPLICABLE,
    OPTION_RETIRED,
    OTHER,
    PRIMARY,
    SECONDARY,
    TERTIARY,
    WEEKLY,
    YEARLY,
)
from django.utils.translation import gettext_lazy as _

from .constants import (
    ACRES,
    ALL_WINDOWS_SCREENED,
    BROTHER_SISTER,
    DECIMALS,
    FAMILY_OWNED,
    GRANDCHILD,
    HECTARES,
    JOINT_OWNED,
    NON_FAMILY_OWNED,
    OWNER,
    PARENT,
    PARENTINLAW,
    SOME_WINDOWS_SCREENED,
    SON_DAUGHTER,
    SON_DAUGHTERINLAW,
    SQ_FEET,
    SQ_METERS,
    WIFE_HUSBAND,
)

EDUCATION_CERTIFICATES_CHOICES = (
    (PRIMARY, _("Primary Certificate")),
    (SECONDARY, _("Secondary Certificate")),
    (TERTIARY, _("post-Secondary/Tertiary/College")),
    (OTHER, _("Other, specify ...")),
    (NOT_APPLICABLE, _("Not applicable, never went to school")),
)

RELATIONSHIP_CHOICES = (
    (WIFE_HUSBAND, _("Wife/Husband")),
    (SON_DAUGHTER, _("Son/Daughter")),
    (SON_DAUGHTERINLAW, _("Son/Daughter-in-law")),
    (GRANDCHILD, _("Grandchild")),
    (PARENT, _("Parent")),
    (PARENTINLAW, _("Parent-in-law")),
    (BROTHER_SISTER, _("Brother/Sister")),
    (OTHER, _("Other, specify ...")),
    (DONT_KNOW, _("Don't know")),
    (NOT_APPLICABLE, _("Not applicable")),
)

EMPLOYMENT_STATUS_CHOICES = (
    ("1", _("Full time employed")),
    ("2", _("Regular part time employed ")),
    ("9", _("Self employed ")),
    ("3", _("Irregular/ occasional/ day worker employment")),
    ("4", _("Non-paid/ voluntary role ")),
    ("5", _("Student")),
    ("6", _("Homemaker")),
    ("7", _("Unemployed (able to work)")),
    ("8", _("Unemployed (unable to work)")),
    (DONT_KNOW, _("Don't know")),
    (NOT_APPLICABLE, _("Not applicable")),
)


MARITAL_CHOICES = (
    ("1", _("Never Married (but not co-habiting)")),
    ("2", _("Co-habiting")),
    ("3", _("Currently Married")),
    ("4", _("Separated/Divorced")),
    ("5", _("Widowed")),
    (OTHER, _("Other, specify ...")),
    (DONT_KNOW, _("Don't know")),
    (NOT_APPLICABLE, _("Not applicable")),
)


RESIDENCE_OWNERSHIP_CHOICES = (
    ("renter", _("Rent")),
    (OWNER, _("Own themselves")),
    (FAMILY_OWNED, _("Owned by someone else in family")),
    (NON_FAMILY_OWNED, _("Owned by someone else other than family member")),
    (JOINT_OWNED, _("Owned together with someone")),
)

WATER_SOURCE_CHOICES = (
    ("piped_into_plot", _("Piped into dwelling/yard plot")),
    ("piped_to_neighbour", _("Piped to neighbour")),
    ("standpipe", _("Public tap/standpipe")),
    ("borehole", _("Tube well or borehole")),
    ("protected_well", _("Protected dug well")),
    ("protected_spring", _("Protected spring")),
    ("rainwater", _("Rainwater")),
    (
        "bottled_water_improved",
        "Bottled water, improved source for cooking/hand washing (1-7)",
    ),
    ("unprotected_well", _("Unprotected dug well")),
    ("unprotected_spring", _("Unprotected spring")),
    ("tanker", _("Tanker truck/cart with small tank")),
    ("surface_water", _("Surface water (river etc.)")),
    (
        "bottled_water_unimproved",
        "Bottle water, unimproved source for cooking/hand washing (9-12)",
    ),
    (OTHER, _("Other, specify ...")),
)


WATER_OBTAIN_CHOICES = (
    ("on_premises", _("Water on premises (includes water piped to a neighbour)")),
    ("less_30min", _("Less than 30 minutes")),
    ("greater_30min", _("30 minutes or longer")),
    (DONT_KNOW, _("Don't know")),
)


TOILET_CHOICES = (
    ("1", _("1. Flush/pour flush to piped sewer system - private")),
    ("2", _("2. Flush/pour flush to septic tank - private ")),
    ("3", _("3. Flush/pour flush to pit latrine - private")),
    ("4", _("4. Ventilated improved pit (VIP) latrine - private ")),
    ("5", _("5. Pit latrine with slab - private")),
    ("6", _("6. Composting toilet - private")),
    ("7", _("7. EcoSan - private")),
    ("8", _("8. Flush/pour flush to piped sewer system - shared")),
    ("9", _("9. Flush/pour flush to septic tank - shared")),
    ("10", _("10. Flush/pour flush to pit latrine - shared")),
    ("11", _("11. Ventilated improved pit (VIP) latrine - shared")),
    ("12", _("12. Pit latrine with slab - shared")),
    ("13", _("13. Composting toilet - shared")),
    ("14", _("14. EcoSan - shared")),
    ("15", _("15. Flush/pour flush not to sewer/septic tank/pit latrine")),
    ("16", _("16. Pit latrine with slab (non-washable)")),
    ("17", _("17. Pit latrine without slab/open pit")),
    ("18", _("18. Bucket")),
    ("19", _("19. Hanging toilet/hanging latrine")),
    ("20", _("20. Open defecation (no facility/bush/field)")),
    (OTHER, _("Other, specify ...")),
)

ROOF_MATERIAL_CHOICES = (
    ("1", _("Thatch, Straw")),
    ("2", _("Mud and poles")),
    ("3", _("Tin")),
    ("4", _("Wood")),
    ("5", _("Iron sheet")),
    ("6", _("Tiles ")),
    ("7", _("Cement")),
    (OTHER, _("Other, specify ...")),
)

EAVES_CHOICES = (
    ("1", _("All eaves closed")),
    ("2", _("All eaves open")),
    ("3", _("Partially closed")),
)

EXTERNAL_WALL_MATERIALS_CHOICES = (
    ("1", _("Thatch, Straw")),
    ("2", _("Mud and poles")),
    ("3", _("Timber")),
    (OPTION_RETIRED, _("Un-burnt bricks")),
    ("5", _("Bricks with mud")),
    ("6", _("Bricks with cement")),
    ("7", _("Cement blocks")),
    ("8", _("Stone")),
    (OTHER, _("Other, specify ...")),
)

WINDOW_MATERIAL_CHOICES = (
    ("1", _("Glass")),
    ("2", _("Bags")),
    ("3", _("Wood")),
    ("4", _("Iron/metal")),
    ("5", _("Screens")),
    ("6", _("No windows")),
    (OTHER, _("Other, specify ...")),
)

WINDOW_SCREENING_CHOICES = (
    (ALL_WINDOWS_SCREENED, _("All windows screened")),
    ("2", _("No windows screened")),
    (SOME_WINDOWS_SCREENED, _("Some windows screened")),
)

WINDOW_SCREENING_TYPE_CHOICES = (
    ("1", _("Wire mesh")),
    ("2", _("Old bednet")),
    ("3", _("No windows screened")),
    ("4", _("No windows")),
    (NOT_APPLICABLE, _("Not applicable")),
)

FLOOR_MATERIALS_CHOICES = (
    ("6", _("Earth, sand")),
    ("7", _("Dung, wood, planks, palm, bamboo")),
    ("8", _("Parquet, polished wood, vinyl, asphalt strips")),
    ("9", _("Ceramic tiles")),
    ("10", _("Cement")),
    ("11", _("Carpet")),
    (OTHER, _("Other, specify ...")),
)

LAND_AREA_UNITS = (
    (HECTARES, _("hectares")),
    (ACRES, _("acres")),
    (DECIMALS, _("decimals (1 decimal = hundredth of an acre)")),
    (SQ_FEET, _("sq. feet (length * width)")),
    (SQ_METERS, _("sq. meters (length * width)")),
    (NOT_APPLICABLE, _("Not applicable")),
)
LIGHTING_CHOICES = (
    ("1", _("Electricity")),
    ("2", _("Paraffin, kerosene or gas lantern ")),
    ("3", _("Firewood")),
    ("4", _("Candle")),
    (OTHER, _("Other, specify ...")),
)

COOKING_FUEL_CHOICES = (
    ("1", _("Electricity")),
    ("2", _("LPG/natural gas/biogas")),
    ("3", _("Kerosene")),
    ("4", _("Charcoal")),
    ("5", _("Wood")),
    ("6", _("Coal/lignite, straw/shrubs/grass. agricultural crop, animal dung")),
    ("7", _("No food cooked in the household")),
    (OTHER, _("Other, specify ...")),
)

INCOME_TIME_ESTIMATE_CHOICES = (
    (WEEKLY, _("as weekly income")),
    (MONTHLY, _("as monthly income")),
    (YEARLY, _("as yearly income")),
    (DONT_KNOW, _("Don't know")),
    (DWTA, _("Don't want to answer")),
    (NOT_APPLICABLE, _("Not applicable")),
)

STATUS = (
    ("1", _("Very good")),
    ("2", _("Good")),
    ("3", _("Moderate")),
    ("4", _("Bad")),
    ("5", _("Very bad")),
    (DWTA, _("Don't want to answer")),
)

FINANCIAL_STATUS = (
    ("1", _("Among most wealthy")),
    ("2", _("Above average ")),
    ("3", _("Average wealth")),
    ("4", _("Below average")),
    ("5", _("Among least wealthy")),
    (DWTA, _("Don't want to answer")),
)

REMITTANCE_CURRENCY_CHOICES = (
    ("USD", _("USD")),
    ("GBP", _("GBP")),
    (OTHER, _("Other, specify ...")),
    (DONT_KNOW, _("Don't know")),
)



from clinicedc_constants import DONT_KNOW, NO, NONE, OTHER
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import INVALID_ERROR, FormValidator


class HealthEconomicsHouseholdHeadFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        if (
            self.cleaned_data.get("hh_minors_count") is not None
            and self.cleaned_data.get("hh_count") is not None
            and self.cleaned_data.get("hh_minors_count") >= self.cleaned_data.get("hh_count")
        ):
            self.raise_validation_error(
                {
                    "hh_minors_count": (
                        f"Invalid. Expected less than {self.cleaned_data.get('hh_count')}."
                    )
                },
                INVALID_ERROR,
            )
        self.applicable_if(NO, field="hoh", field_applicable="hoh_gender")
        self.required_if(
            NO,
            field="hoh",
            field_required="hoh_age",
            field_required_evaluate_as_int=True,
        )
        self.applicable_if(NO, field="hoh", field_applicable="relationship_to_hoh")
        self.validate_other_specify(field="relationship_to_hoh")
        self.applicable_if(NO, field="hoh", field_applicable="hoh_religion")
        self.validate_other_specify(field="hoh_religion")
        self.applicable_if(NO, field="hoh", field_applicable="hoh_ethnicity")
        self.validate_other_specify(field="hoh_ethnicity")
        self.applicable_if(NO, field="hoh", field_applicable="hoh_education")
        self.validate_other_specify(field="hoh_education")
        self.applicable_if(NO, field="hoh", field_applicable="hoh_employment_status")
        self.validate_other_specify(field="hoh_employment_status")
        self.applicable_if(NO, field="hoh", field_applicable="hoh_employment_type")
        self.validate_other_specify(field="hoh_employment_type")
        self.applicable_if(NO, field="hoh", field_applicable="hoh_marital_status")
        self.validate_other_specify(field="hoh_marital_status")
        self.m2m_applicable_if(NO, field="hoh", m2m_field="hoh_insurance")
        self.m2m_single_selection_if(DONT_KNOW, NONE, m2m_field="hoh_insurance")
        self.m2m_other_specify(
            OTHER, m2m_field="hoh_insurance", field_other="hoh_insurance_other"
        )



from clinicedc_constants import MONTHLY, WEEKLY, YEARLY, YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator


class HealthEconomicsIncomeFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        for fld in [
            "wages",
            "selling",
            "rental_income",
            "pension",
            "ngo_assistance",
            "interest",
            "internal_remit",
            "external_remit",
        ]:
            self.applicable_if(YES, field=fld, field_applicable=f"{fld}_value_known")
            self.required_if(
                WEEKLY,
                MONTHLY,
                YEARLY,
                field=f"{fld}_value_known",
                field_required=f"{fld}_value",
            )

        self.required_if_true(
            self.cleaned_data.get("external_remit_value"),
            field_required="external_remit_currency",
        )
        self.validate_other_specify(field="external_remit_currency")
        self.required_if(YES, field="more_sources", field_required="more_sources_other")
        self.required_if(YES, field="household_debt", field_required="household_debt_value")



# Generated by Django 5.2.6 on 2025-09-30 23:23

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0008_healtheconomics_alter_education_extra_value_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Education",
            new_name="EducationType",
        ),
        migrations.RenameModel(
            old_name="HistoricalEducation",
            new_name="HistoricalEducationType",
        ),
        migrations.DeleteModel(
            name="HealthEconomics",
        ),
        migrations.AlterModelOptions(
            name="educationtype",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Education type",
                "verbose_name_plural": "Education types",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaleducationtype",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Education type",
                "verbose_name_plural": "historical Education types",
            },
        ),
        migrations.RenameIndex(
            model_name="educationtype",
            new_name="edc_he_educ_display_73adba_idx",
            old_name="edc_he_educ_display_66a8e9_idx",
        ),
    ]



from __future__ import annotations

from clinicedc_constants import YES
from django.core.exceptions import ObjectDoesNotExist

from ..utils import (
    get_assets_model_cls,
    get_household_head_model_cls,
    get_income_model_cls,
    get_patient_model_cls,
    get_property_model_cls,
)


class Predicates:
    def is_required_by_date(self, visit, **kwargs) -> bool:  # noqa: ARG002
        return True

    @staticmethod
    def get_household_head(visit):
        try:
            obj = get_household_head_model_cls().objects.get(
                subject_visit__subject_identifier=visit.subject_identifier
            )
        except ObjectDoesNotExist:
            obj = None
        return obj

    def household_head_required(self, visit, **kwargs) -> bool:  # noqa: ARG002
        return (
            self.is_required_by_date(visit)
            and not get_household_head_model_cls()
            .objects.filter(subject_visit__subject_identifier=visit.subject_identifier)
            .exists()
        )

    def patient_required(self, visit, **kwargs) -> bool:  # noqa: ARG002
        required = False
        hoh_obj = self.get_household_head(visit)
        if hoh_obj and (
            not get_patient_model_cls()
            .objects.filter(subject_visit__subject_identifier=visit.subject_identifier)
            .exists()
        ):
            required = hoh_obj.hoh == YES and self.is_required_by_date(visit)

        return required

    def assets_required(self, visit, **kwargs) -> bool:  # noqa: ARG002
        return (
            self.is_required_by_date(visit)
            and not get_assets_model_cls()
            .objects.filter(subject_visit__subject_identifier=visit.subject_identifier)
            .exists()
        )

    def property_required(self, visit, **kwargs) -> bool:  # noqa: ARG002
        return (
            self.is_required_by_date(visit)
            and not get_property_model_cls()
            .objects.filter(subject_visit__subject_identifier=visit.subject_identifier)
            .exists()
        )

    def income_required(self, visit, **kwargs) -> bool:  # noqa: ARG002
        return (
            self.is_required_by_date(visit)
            and not get_income_model_cls()
            .objects.filter(subject_visit__subject_identifier=visit.subject_identifier)
            .exists()
        )



from django.utils.translation import gettext_lazy as _
from edc_crf.model_mixins import CrfModelMixin, SingletonCrfModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import PatientModelMixin


class HealthEconomicsPatient(
    SingletonCrfModelMixin,
    PatientModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = _("Health Economics: Patient")
        verbose_name_plural = _("Health Economics: Patient")
        indexes = (*CrfModelMixin.Meta.indexes, *BaseUuidModel.Meta.indexes)



from edc_list_data.model_mixins import ListModelManager, ListModelMixin
from edc_model.models import HistoricalRecords


class EducationType(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Education type"
        verbose_name_plural = "Education types"


class EmploymentType(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Employment type"
        verbose_name_plural = "Employment types"


class Ethnicities(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Ethnicities"
        verbose_name_plural = "Ethnicities"


class InsuranceTypes(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Insurance Types"
        verbose_name_plural = "Insurance Types"


class Nationalities(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Nationalities"
        verbose_name_plural = "Nationalities"


class Religions(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Religions"
        verbose_name_plural = "Religions"



from django import forms
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidatorMixin
from edc_form_validators.form_validator import FormValidator

from edc_he.form_validators import SimpleFormValidatorMixin

from ..models import Education


class EducationFormValidator(CrfFormValidatorMixin, SimpleFormValidatorMixin, FormValidator):
    def clean(self) -> None:
        self.clean_education()


class EducationForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = EducationFormValidator

    class Meta:
        model = Education
        fields = "__all__"



from edc_metadata import NOT_REQUIRED, REQUIRED
from edc_metadata.metadata_rules import CrfRule, CrfRuleGroup


class HealthEconomicsRuleGroup(CrfRuleGroup):
    hoh = CrfRule(
        predicate="household_head_required",
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["healtheconomicshouseholdhead"],
    )

    patient = CrfRule(
        predicate="patient_required",
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["healtheconomicspatient"],
    )

    assets = CrfRule(
        predicate="assets_required",
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["healtheconomicsassets"],
    )

    property = CrfRule(
        predicate="property_required",
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["healtheconomicsproperty"],
    )

    income = CrfRule(
        predicate="income_required",
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["healtheconomicsincome"],
    )

    class Meta:
        abstract = True



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin, CrfSingletonModelFormMixin

from ..form_validators import HealthEconomicsPropertyFormValidator
from ..models import HealthEconomicsProperty
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsPropertyForm(
    CrfSingletonModelFormMixin,
    HealthEconomicsModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HealthEconomicsPropertyFormValidator

    class Meta:
        model = HealthEconomicsProperty
        fields = "__all__"



from .assets_model_admin_mixin import HealthEconomicsAssetsModelAdminMixin
from .household_head_model_admin_mixin import (
    HealthEconomicsHouseholdHeadModelAdminMixin,
)
from .income_model_admin_mixin import HealthEconomicsIncomeModelAdminMixin
from .patient_model_admin_mixin import HealthEconomicsPatientModelAdminMixin
from .property_model_admin_mixin import HealthEconomicsPropertyModelAdminMixin



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_he",
            "0006_alter_education_options_alter_employmenttype_options_and_more",
        ),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="education",
            name="edc_he_educ_name_64c0c9_idx",
        ),
        migrations.RemoveIndex(
            model_name="employmenttype",
            name="edc_he_empl_name_b36be4_idx",
        ),
        migrations.RemoveIndex(
            model_name="ethnicities",
            name="edc_he_ethn_name_e7e98d_idx",
        ),
        migrations.RemoveIndex(
            model_name="insurancetypes",
            name="edc_he_insu_name_f9caac_idx",
        ),
        migrations.RemoveIndex(
            model_name="nationalities",
            name="edc_he_nati_name_cba5f6_idx",
        ),
        migrations.RemoveIndex(
            model_name="religions",
            name="edc_he_reli_name_b2fcb4_idx",
        ),
    ]



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin, CrfSingletonModelFormMixin

from ..form_validators import HealthEconomicsIncomeFormValidator
from ..models import HealthEconomicsIncome
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsIncomeForm(
    CrfSingletonModelFormMixin,
    HealthEconomicsModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HealthEconomicsIncomeFormValidator

    class Meta:
        model = HealthEconomicsIncome
        fields = "__all__"
        help_texts = {"external_remit_value": ""}  # noqa: RUF012



from __future__ import annotations

from clinicedc_constants import NO
from django.db import models
from django.utils.translation import gettext_lazy as _
from edc_constants.choices import YES_NO

default_field_data = {
    "solar_panels": _("Solar panels"),
    "radio": _("Radio"),
    "television": _("Television"),
    "mobile_phone": _("Mobile phone"),
    "computer": _("Computer"),
    "telephone": _("Non-mobile telephone"),
    "fridge": _("Fridge"),
    "generator": _("Generator"),
    "iron": _("Flat iron"),
    "bicycle": _("Bicycle"),
    "motorcycle": _("Motorcycle/scooter (PikiPiki/Boda Boda)"),
    "dala_dala": _("Dala Dala"),
    "car": _("Car"),
    "motorboat": _("Boat with a motor"),
    "large_livestock": _("Large Livestock (e.g. cows, bulls, other cattle, horses, donkeys)"),
    "small_animals": _("Small animals (goats, sheep, chickens or other poultry, etc)"),
    "shop": _("A business or shop"),
}


def assets_model_mixin_factory(field_data: dict[str, str] | None = None):
    field_data = field_data or default_field_data

    class AbstractModel(models.Model):
        class Meta:
            abstract = True

    opts = {}
    for field_name, prompt in field_data.items():
        opts.update(
            {
                field_name: models.CharField(
                    verbose_name=prompt,
                    max_length=15,
                    choices=YES_NO,
                    default=NO,
                ),
            }
        )
    for fld_name, fld_cls in opts.items():
        AbstractModel.add_to_class(fld_name, fld_cls)

    return AbstractModel



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.conf import settings

if TYPE_CHECKING:
    from .models import (
        HealthEconomicsAssets,
        HealthEconomicsHouseholdHead,
        HealthEconomicsIncome,
        HealthEconomicsPatient,
        HealthEconomicsProperty,
    )


def get_household_head_model() -> str:
    return getattr(
        settings, "EDC_HE_HOUSEHOLDHEAD_MODEL", "edc_he.healtheconomicshouseholdhead"
    )


def get_household_head_model_cls() -> type[HealthEconomicsHouseholdHead]:
    return django_apps.get_model(get_household_head_model())


def get_patient_model() -> str:
    return getattr(settings, "EDC_HE_PATIENT_MODEL", "edc_he.healtheconomicspatient")


def get_patient_model_cls() -> type[HealthEconomicsPatient]:
    return django_apps.get_model(get_patient_model())


def get_property_model() -> str:
    return getattr(settings, "EDC_HE_PROPERTY_MODEL", "edc_he.healtheconomicsproperty")


def get_property_model_cls() -> type[HealthEconomicsProperty]:
    return django_apps.get_model(get_property_model())


def get_income_model() -> str:
    return getattr(settings, "EDC_HE_INCOME_MODEL", "edc_he.healtheconomicsincome")


def get_income_model_cls() -> type[HealthEconomicsIncome]:
    return django_apps.get_model(get_income_model())


def get_assets_model() -> str:
    return getattr(settings, "EDC_HE_ASSETS_MODEL", "edc_he.healtheconomicsassets")


def get_assets_model_cls() -> type[HealthEconomicsAssets]:
    return django_apps.get_model(get_assets_model())



from django.contrib import admin
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple


class HealthEconomicsPropertyModelAdminMixin:
    form = None

    additional_instructions = _(
        "We want to learn about the household and we use these questions "
        "to get an understanding of wealth and opportunities in the community. "
    )
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Property",
            {
                "description": format_html(
                    "{html}",
                    html=mark_safe(  # noqa: S308
                        render_to_string("edc_he/property/description.html"),
                    ),
                ),
                "fields": (
                    "land_owner",
                    "land_value",
                    "land_surface_area",
                    "land_surface_area_units",
                    "land_additional",
                    "land_additional_value",
                ),
            },
        ),
        (
            "Calculated values",
            {
                "description": "To be calculated (or recalculated) when this form is saved",
                "classes": ("collapse",),
                "fields": ("calculated_land_surface_area",),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    readonly_fields = ("calculated_land_surface_area",)

    radio_fields = {  # noqa: RUF012
        "land_owner": admin.VERTICAL,
        "land_additional": admin.VERTICAL,
        "land_surface_area_units": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }



# Generated by Django 4.2.7 on 2023-12-03 00:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0005_rename_employment_employmenttype_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="education",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Education",
                "verbose_name_plural": "Education",
            },
        ),
        migrations.AlterModelOptions(
            name="employmenttype",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Employment type",
                "verbose_name_plural": "Employment types",
            },
        ),
        migrations.AlterModelOptions(
            name="ethnicities",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Ethnicities",
                "verbose_name_plural": "Ethnicities",
            },
        ),
        migrations.AlterModelOptions(
            name="insurancetypes",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Insurance Types",
                "verbose_name_plural": "Insurance Types",
            },
        ),
        migrations.AlterModelOptions(
            name="nationalities",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Nationalities",
                "verbose_name_plural": "Nationalities",
            },
        ),
        migrations.AlterModelOptions(
            name="religions",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Religions",
                "verbose_name_plural": "Religions",
            },
        ),
        migrations.RemoveIndex(
            model_name="education",
            name="edc_he_educ_id_52260a_idx",
        ),
        migrations.RemoveIndex(
            model_name="employmenttype",
            name="edc_he_empl_id_810a9e_idx",
        ),
        migrations.RemoveIndex(
            model_name="ethnicities",
            name="edc_he_ethn_id_c14a61_idx",
        ),
        migrations.RemoveIndex(
            model_name="insurancetypes",
            name="edc_he_insu_id_79fd66_idx",
        ),
        migrations.RemoveIndex(
            model_name="nationalities",
            name="edc_he_nati_id_9a6ed2_idx",
        ),
        migrations.RemoveIndex(
            model_name="religions",
            name="edc_he_reli_id_e5b168_idx",
        ),
        migrations.AlterField(
            model_name="education",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="education",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="education",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="employmenttype",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="employmenttype",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="employmenttype",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicities",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicities",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicities",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleducation",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="historicalemploymenttype",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="historicalethnicities",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="historicalinsurancetypes",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnationalities",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreligions",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="insurancetypes",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="insurancetypes",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="insurancetypes",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="nationalities",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="nationalities",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="nationalities",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="religions",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="religions",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="religions",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AddIndex(
            model_name="education",
            index=models.Index(fields=["name"], name="edc_he_educ_name_64c0c9_idx"),
        ),
        migrations.AddIndex(
            model_name="education",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_he_educ_display_66a8e9_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="employmenttype",
            index=models.Index(fields=["name"], name="edc_he_empl_name_b36be4_idx"),
        ),
        migrations.AddIndex(
            model_name="employmenttype",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_he_empl_display_94ff23_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="ethnicities",
            index=models.Index(fields=["name"], name="edc_he_ethn_name_e7e98d_idx"),
        ),
        migrations.AddIndex(
            model_name="ethnicities",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_he_ethn_display_872238_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="insurancetypes",
            index=models.Index(fields=["name"], name="edc_he_insu_name_f9caac_idx"),
        ),
        migrations.AddIndex(
            model_name="insurancetypes",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_he_insu_display_751194_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="nationalities",
            index=models.Index(fields=["name"], name="edc_he_nati_name_cba5f6_idx"),
        ),
        migrations.AddIndex(
            model_name="nationalities",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_he_nati_display_178bbe_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="religions",
            index=models.Index(fields=["name"], name="edc_he_reli_name_b2fcb4_idx"),
        ),
        migrations.AddIndex(
            model_name="religions",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_he_reli_display_11bcd9_idx",
            ),
        ),
    ]



from clinicedc_constants import NOT_APPLICABLE
from django.contrib import admin
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple


class HealthEconomicsHouseholdHeadModelAdminMixin:
    form = None

    additional_instructions = format_html(
        "{html}",
        html=mark_safe(
            "<H5><B><font color='orange'>Interviewer to read</font></B></H5>"
            "<p>We want to learn about the household and we use these questions "
            "to get an understanding of wealth and opportunities in the community.</p>"
        ),  # nosec B703, B308
    )

    household_description = _(
        "we mean a person or persons (people/ members) "
        "who share the same kitchen (pot), live together, and run the household "
        "expenditure from the same income is known as a household"
    )
    household_member_description = _(
        "we mean a person identified on "
        "the basis that they shared a place of living together most of time for "
        "the past one year"
    )

    household_description_extra = _(
        "When it is difficult to demarcate "
        "'most of the time', living together for the past six months or more "
        "should be used to find out whether or not the person is a "
        "household member"
    )

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Household members",
            {
                "description": format_html(
                    "{html}",
                    html=mark_safe(  # noqa: S308
                        render_to_string(
                            "edc_he/household_head/household_members_description.html",
                            context=dict(
                                household_description=household_description,
                                household_member_description=household_member_description,
                                household_description_extra=household_description_extra,
                            ),
                        )
                    ),  # nosec B703, B308
                ),
                "fields": (
                    "hh_count",
                    "hh_minors_count",
                ),
            },
        ),
        (
            "Household head: Gender and age",
            {
                "description": "",
                "fields": (
                    "hoh",
                    "hoh_gender",
                    "hoh_age",
                ),
            },
        ),
        (
            "Head of household",
            {
                "description": format_html(
                    "{html}",
                    html=mark_safe(  # noqa: S308
                        render_to_string(
                            "edc_he/household_head/household_head_description.html"
                        )
                    ),  # nosec B703, B308
                ),
                "fields": (
                    "relationship_to_hoh",
                    "relationship_to_hoh_other",
                ),
            },
        ),
        (
            "Head of household: Religion",
            {
                "description": "",
                "fields": (
                    "hoh_religion",
                    "hoh_religion_other",
                ),
            },
        ),
        (
            "Head of household: Ethnicity",
            {
                "description": "",
                "fields": (
                    "hoh_ethnicity",
                    "hoh_ethnicity_other",
                ),
            },
        ),
        (
            "Head of household: Education",
            {
                "description": "",
                "fields": (
                    "hoh_education",
                    "hoh_education_other",
                ),
            },
        ),
        (
            "Head of household: Employment",
            {
                "description": "",
                "fields": (
                    "hoh_employment_status",
                    "hoh_employment_type",
                    "hoh_employment_type_other",
                ),
            },
        ),
        (
            "Head of household: Marital status",
            {
                "description": "",
                "fields": (
                    "hoh_marital_status",
                    "hoh_marital_status_other",
                ),
            },
        ),
        (
            "Head of household: Insurance",
            {
                "description": "",
                "fields": (
                    "hoh_insurance",
                    "hoh_insurance_other",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {  # noqa: RUF012
        "hoh": admin.VERTICAL,
        "relationship_to_hoh": admin.VERTICAL,
        "hoh_gender": admin.VERTICAL,
        "hoh_religion": admin.VERTICAL,
        "hoh_ethnicity": admin.VERTICAL,
        "hoh_education": admin.VERTICAL,
        "hoh_employment_status": admin.VERTICAL,
        "hoh_employment_type": admin.VERTICAL,
        "hoh_marital_status": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }

    filter_horizontal = ("hoh_insurance",)

    def get_changeform_initial_data(self, request):
        defaults = super().get_changeform_initial_data(request)
        for fld in self.model._meta.get_fields():
            if fld.name in self.filter_horizontal:
                qs = fld.related_model.objects.filter(name=NOT_APPLICABLE)
                defaults.update({fld.name: qs})
            elif fld.name in self.radio_fields:
                if fld.related_model:
                    try:
                        value = fld.related_model.objects.get(name=NOT_APPLICABLE)
                    except AttributeError as e:
                        if "related_model" not in str(e):
                            raise
                    else:
                        defaults.update({fld.name: value})
        return defaults



from django.contrib import admin
from edc_crf.modeladmin_mixins import CrfModelAdmin

from ..admin_site import edc_he_admin
from ..forms import HealthEconomicsPropertyForm
from ..modeladmin_mixins import HealthEconomicsPropertyModelAdminMixin
from ..models import HealthEconomicsProperty


@admin.register(HealthEconomicsProperty, site=edc_he_admin)
class HealthEconomicsPropertyAdmin(HealthEconomicsPropertyModelAdminMixin, CrfModelAdmin):
    form = HealthEconomicsPropertyForm
